require 'json'

module UU
  module OS
    module Application

      # Special type of exception to be raised anywhere in controller
      # implementation (including overridden system request processing
      # methods) which causes interruption of processing and immediate
      # return of given response.
      # @note Not intended for regular use. Use should be avoided in
      #    common control methods!
      class ImmediateResponse < Exception

        # Creates new instance of immediate response exception.
        # @param [Fixnum] status HTTP status code (see {http://en.wikipedia.org/wiki/List_of_HTTP_status_codes
        #    List of HTTP status codes} for details).
        # @param [Hash, UU::OS::HTTP::HTTPHeaders] headers Response headers (multi-value headers should
        #    be given as array value).
        # @param [String, IO, Exception] body Response body.
        def initialize(status, headers, body)
          @status = status.to_i
          @headers = headers || {}
          @headers = @headers.inject({}) do |tmp, (key, value)|
            key = key.to_s.gsub('_', '-').downcase
            if value.kind_of?(Array)
              tmp[key] = value.join(',')
            else
              tmp[key] = value.to_s
            end
            tmp
          end
          if body.kind_of?(Exception)
            @body = body.to_json
            @headers['content-type'] ||= 'application/json'
          else
            @body = body
          end
        end

        # Returns response object.
        # @return [Object] Response object to be handled by server.
        def response
          {status: @status, headers: @headers, body: @body}
        end

      end

    end
  end
end
